import logging
from typing import Dict, Any, List, Optional
from telegram import Update
from telegram.ext import ContextTypes
from .base import InfoCommandsBase

logger = logging.getLogger(__name__)

class MarketCommands(InfoCommandsBase):
    """Handles all market-related commands."""

    async def market_command(self, update: Update, context: ContextTypes.DEFAULT_TYPE) -> None:
        """Handle the /market command."""
        try:
            if not self._is_authorized(update):
                await self._reply(update, "❌ Unauthorized access.")
                return

            # Get market data
            market_data = self.trading_engine.get_market_data()
            if not market_data:
                await self._reply(update, "❌ Market data not available.")
                return

            # Format market text
            market_text = "📊 <b>Market Overview</b>\n\n"

            for symbol, data in market_data.items():
                try:
                    base_asset = symbol.split('/')[0] if '/' in symbol else symbol.split(':')[0]
                    price = float(data.get('price', 0))
                    volume_24h = float(data.get('volume_24h', 0))
                    change_24h = float(data.get('change_24h', 0))
                    high_24h = float(data.get('high_24h', 0))
                    low_24h = float(data.get('low_24h', 0))

                    # Format market details
                    formatter = self._get_formatter()
                    price_str = formatter.format_price_with_symbol(price, base_asset)
                    volume_str = formatter.format_amount(volume_24h, base_asset)
                    high_str = formatter.format_price_with_symbol(high_24h, base_asset)
                    low_str = formatter.format_price_with_symbol(low_24h, base_asset)

                    # Market header
                    change_emoji = "🟢" if change_24h >= 0 else "🔴"
                    market_text += f"{change_emoji} <b>{base_asset}</b>\n"
                    market_text += f"   💰 Price: {price_str}\n"
                    market_text += f"   📈 24h Change: {change_24h:+.2f}%\n"
                    market_text += f"   📊 24h Volume: {volume_str} {base_asset}\n"
                    market_text += f"   🔺 24h High: {high_str}\n"
                    market_text += f"   🔻 24h Low: {low_str}\n\n"

                except Exception as e:
                    logger.error(f"Error processing market data for {symbol}: {e}")
                    continue

            await self._reply(update, market_text.strip())

        except Exception as e:
            logger.error(f"Error in market command: {e}")
            await self._reply(update, "❌ Error retrieving market data.")
